@@ -100,14 +100,14 @@ class FileProcessorFacade:
         else:
             raise ValueError('unsupport file type')
 
-    def write(self, df, io, sheet_name='Sheet1'):
+    def write(self, df, io, sheet_name='Sheet1', index=False):
         file = FileName(io)
         if file.is_ext(['.xlsx', '.xls']):
             file.make_dir()
-            self.__excel_model.write(df, file.fullpath, sheet_name=sheet_name)
+            self.__excel_model.write(df, file.fullpath, sheet_name=sheet_name, index=index)
         elif file.is_ext('.csv'):
             file.make_dir()
-            self.__csv_model.write(df, file.fullpath)
+            self.__csv_model.write(df, file.fullpath, index=index)
         else:
             raise ValueError('unsupport file type')
 
