@@ -408,6 +408,119 @@ def energy_consumption_test(df, factor, t, current, voltage, last_only=True):
     return get_integral(df[['t', 'power']], last_only=last_only) / 3600000
 
 
+@validate_df
+def filter_by_fixed_interval(df, column, start, end, step=5, precision=0.2):
+    """
+    按固定步长，筛选数据中的部分数据。常用于动力性等试验中提取以5为间隔的车速点。
+    筛选规则如下：由上往下依次遍历df，目标列数据首次超过目标点时，比较该点与上一点与目标点之间的距离，取较小的作为结果
+    :param df:
+    :param column: 用于筛选的数据列
+    :param start: 起始数值
+    :param end: 结束数值
+    :param step: 步长，必须能被start-end整除
+    :param precision: 数据点精度
+    :return:
+    """
+    # 参数判断
+    # if not(isinstance(start, int) and isinstance(end, int) and isinstance(step, int)):
+    #     raise TypeError('param start,end and step must be integer')
+    if start == end:
+        raise ValueError('param start and end cannot be equal')
+    if (end - start) % step:
+        raise ValueError('param step cannot be divisible')
+
+    # 截取起始数据段
+    df = df.reset_index(drop=False)
+    start_index = df.loc[(df[column] - start).abs().argsort()].index[0]
+    end_index = df.loc[(df[column] - end).abs().argsort()].index[0]
+    df = df.loc[start_index: end_index]
+    df = df.reset_index(drop=True)
+
+    # 判断起始数据精度是否达标
+    if abs(df[column].iloc[0] - start) > precision:
+        raise RuntimeError('there is no column satisfying the precision of start')
+    if abs(df[column].iloc[-1] - end) > precision:
+        raise RuntimeError('there is no column satisfying the precision of end')
+
+    # 查找各目标点
+    res = df.iloc[[0, -1]]
+    length = len(df) - 2
+    pointer = 0
+    step = abs(step) if start < end else -abs(step)
+
+    for i in range(abs(round((end - start) / step))):
+        target = (i + 1) * step + start
+        while True:
+            pointer = pointer + 1
+            if pointer > length:
+                break
+            if (start < end and df[column][pointer] > target)\
+                    or (start > end and df[column][pointer] < target):
+                before = abs(df[column][pointer - 1] - target)
+                after = abs(df[column][pointer] - target)
+                if before > precision and after > precision:
+                    raise RuntimeError('{} cannot satisfy the precision'.format(target))
+                if after < before:
+                    res = res.append(df.loc[pointer])
+                else:
+                    res = res.append(df.loc[pointer - 1])
+                break
+
+    # 判断目标点是否全部找到
+    if len(res) != (abs(round((end - start) / step)) + 1):
+        raise RuntimeError('can not find all target')
+
+    res = res.sort_values(by=column)
+    res = res.set_index(res['index'].values).drop(columns=['index'])
+    res.index = res.index.astype(int)
+
+    return res
+
+
+@validate_df
+def brake_test(df, t, v_x, s=None):
+    """
+    计算制动试验结果，主要参考：
+    《GB 12676-2014 商用车辆和挂车制动系统技术要求及试验方法》 5.1.1.3
+    :param df:
+    :param t: 时间列名称
+    :param v_x: 前进车速列名称
+    :param s: 行驶距离列名称，为None则自动根据时间车速积分
+    :return:
+    """
+    # 提取并重命名相关数据列
+    if s is None:
+        df = df[[t, v_x]]
+        df = df.rename(columns={t: 't', v_x: 'v_x'})
+        df['s'] = get_integral(df[['t', 'v_x']]) / 3.6
+    else:
+        df = df[[t, v_x, s]]
+        df = df.rename(columns={t: 't', v_x: 'v_x', s: 's'})
+
+    # 计算起始车速和结束车速
+    start_speed = df.v_x.iloc[0]
+    # end_speed = df.v_x.iloc[-1]
+
+    # 计算起始距离和结束距离
+    start_s = df.s.iloc[0]
+    end_s = df.s.iloc[-1]
+
+    f = interpolate.interp1d(df.v_x, df.s)
+    vb = 0.8 * start_speed
+    ve = 0.1 * start_speed
+    sb = f(vb) - start_s
+    se = f(ve) - start_s
+
+    # 计算充分发出的平均减速度
+    mfdd = (vb**2 - ve**2) / 25.92 / (se - sb)
+
+    # 计算制动时间和制动距离
+    brake_t = df.t.iloc[-1] - df.t.iloc[0]
+    brake_s = end_s - start_s
+
+    return mfdd, start_speed, brake_t, brake_s
+
+
 if __name__ == '__main__':
     # p = driving_resistance(12500)
     # print(p)
